# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-04-12 08:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_multitenant.fields
import django_multitenant.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0007_alter_foreignkey_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='somerelatedmodel',
            name='opened',
            field=models.BooleanField(default=True),
        ),
    ]
