# Generated by Django 2.2.9 on 2020-01-29 03:57

from django.db import migrations, models
import django.db.models.deletion
import django_multitenant.fields


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0018_auto_20200128_0902'),
    ]

    atomic = False
    operations = [
        migrations.AddField(
            model_name='account',
            name='employee',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts', to='tests.Employee'),
        ),
    ]
