# Generated by Django 2.1.1 on 2019-03-01 08:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0004_auto_20190301_0834'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tenantnotidmodel',
            name='tenant_column',
            field=models.IntegerField(editable=False, primary_key=True, serialize=False),
        ),
    ]
